import logging
import inquirer
from pathlib import Path
from dotenv import set_key, dotenv_values
from typing import Optional


ENV_FILE_PATH = Path(".env").resolve()


def setup_logging() -> None:
    logging.basicConfig(level=logging.INFO, format="%(message)s")


def check_if_user_wants_to_override_env_file() -> bool:
    question = [
        inquirer.List(
            "choice",
            message="Existing user settings were found, do you want to overwrite this?",
            choices=["Yes", "No"],
            default="No",
        )
    ]
    response = inquirer.prompt(question)

    return response["choice"] == "Yes"


def save_user_info_to_env() -> None:
    if not ENV_FILE_PATH.is_file() or check_if_user_wants_to_override_env_file():
        get_user_info_api_key()
        get_user_info_directories()
        logging.info("User setting saved to '.env' file")


def get_user_info_directories() -> None:
    output_folder = get_user_output_folder()
    input_folders = get_user_input_folders()

    input_folders_str = ",".join(input_folders)

    set_key(
        dotenv_path=ENV_FILE_PATH,
        key_to_set="INPUT_FOLDERS",
        value_to_set=input_folders_str,
        quote_mode="never"
    )
    set_key(
        dotenv_path=ENV_FILE_PATH,
        key_to_set="OUTPUT_FOLDER",
        value_to_set=output_folder,
        quote_mode="never"
    )


def prompt_user_for_info(
    info_required: str,
    prompt_message: str,
    logging_message: Optional[str] = None,
) -> str:

    if logging_message:
        logging.info(logging_message)

    prompt = [inquirer.Text(info_required, message=prompt_message)]
    response = inquirer.prompt(prompt)[info_required]

    return response


def get_user_output_folder() -> str:
    """
    Ask user for output folder, validate it can be converted to a Path object
    and it exists, then return as a string
    """
    logging_message = (
        "\n\nPlease enter the output folder (where the film list CSV should be saved)\n"
        "e.g. C:/Media or /home/username/media\n"
    )
    prompt_message = "Output folder"

    response = prompt_user_for_info("output_folder", prompt_message, logging_message)
    output_folder_path = return_as_path_if_valid(response)

    return str(output_folder_path)


def get_user_input_folders() -> list[str]:
    """
    Ask user for one or more input volders, validate they can be converted to Path objects
    and they exist, then return as list of strings
    """
    logging_message = (
        "\n\nPlease enter one or more input folders that contain your films.\n"
        "Multiple folders should be separated with a comma e.g. C:/Films,E:/Films\n"
    )
    prompt_message = "Input folder(s)"

    response = prompt_user_for_info("input_folders", prompt_message, logging_message)

    input_folders = response.split(",")

    valid_input_folder_paths = [
        return_as_path_if_valid(folder) for folder in input_folders
    ]

    valid_input_folders_str = [str(folder).rstrip("\\") for folder in valid_input_folder_paths]

    return valid_input_folders_str


def return_as_path_if_valid(folder_path_str: Optional[str]) -> Path:
    if not folder_path_str:
        raise ValueError("No folder path was selected in the dialog box, please re-run")

    folder_path_str_formatted = folder_path_str.strip()

    # Enable ~ shorthand to be a valid path and resolve the absolute path
    folder_path = Path(folder_path_str_formatted).expanduser().resolve()

    if not folder_path.is_dir():
        raise ValueError(
            f"Folder path '{folder_path_str}' does not exist. Please try a different path."
        )

    return folder_path


def get_user_info_api_key() -> None:
    message = "Please enter your API key from ANT"
    api_key = prompt_user_for_info("api_key", message)

    if api_key:
        set_key(
            dotenv_path=ENV_FILE_PATH,
            key_to_set="API_KEY",
            value_to_set=api_key,
            quote_mode="never"
        )
    else:
        raise ValueError("No API key was set - please restart")


def load_env_file() -> tuple[str, list[Path], Path]:
    settings = dotenv_values(ENV_FILE_PATH)

    expected_settings = ["API_KEY", "INPUT_FOLDERS", "OUTPUT_FOLDER"]
    missing_settings = [
        setting for setting in expected_settings if settings.get(setting) is None
    ]

    if missing_settings:
        raise ValueError(
            f"The following user settings were not found in the '.env' file: {','.join(missing_settings)}"
            "\nPlease restart"
        )

    api_key = settings["API_KEY"]

    input_folders_str = settings["INPUT_FOLDERS"].split(",")
    input_folders = [Path(folder) for folder in input_folders_str]

    output_folder = Path(settings["OUTPUT_FOLDER"])

    logging.info("User settings loaded from '.env' file\n")

    return api_key, input_folders, output_folder



import pandas as pd
import numpy as np
from rebulk.match import MatchesDict
import logging
from guessit import guessit
from pathlib import Path
import re
import sys
import shutil


class FilmProcessor:
    def __init__(self, input_folders: list[str], output_folder: str):
        self.file_extensions: list[str] = ["mp4", "avi", "mkv", "mpeg", "m2ts"]
        self.input_folders: list[str] = input_folders
        self.output_folder: Path = Path(output_folder)
        self.film_list_df_types: dict[str, str] = {
            "Full file path": "string",
            "Parsed film title": "string",
            "Film size (GB)": "float64",
            "Resolution": "string",
            "Codec": "string",
            "Source": "string",
            "Release group": "string",
            "Already on ANT?": "string",
            "Info": "string",
        }
        self.csv_file_path: Path = self.output_folder / "Film list.csv"
        self.backup_csv_file_path: Path = (
            self.output_folder / "Film list old version backup.csv"
        )

    def get_film_file_paths(self) -> list[Path]:
        """
        Scan the input folder from parameters for the given file extensions,
        adding any files to a list.
        """
        paths = []
        for ext in self.file_extensions:
            for folder in self.input_folders:
                paths.extend(Path(folder).glob(f"**/*.{ext}"))

        filtered_paths = self.remove_paths_containing_extras_folder(paths)
        cleaned_paths = self.remove_paths_if_unopenable(filtered_paths)

        if not cleaned_paths:
            raise ValueError(
                "No films were found, check the INPUT_FOLDERS value in parameters.py"
            )

        cleaned_paths.sort()

        return cleaned_paths

    def get_film_info_from_file_paths(
        self, film_file_paths: list[Path]
    ) -> pd.DataFrame:
        film_sizes = self.get_film_sizes_from_file_paths(film_file_paths)
        guessed_films = self.get_guessit_info_from_film_paths(film_file_paths)
        film_titles = self.get_formatted_titles_from_guessed_films(guessed_films)
        film_resolutions = self.get_film_resolutions_from_guessed_films(guessed_films)
        film_codecs = self.get_codec_from_guessed_films(guessed_films)
        film_sources = self.get_source_from_guessed_films(guessed_films)
        release_groups = self.get_release_groups_from_guessed_films(guessed_films)

        film_list_df = self.create_film_list_dataframe(
            film_file_paths,
            film_sizes,
            film_titles,
            film_resolutions,
            film_codecs,
            film_sources,
            release_groups,
        )

        return film_list_df

    def combine_with_existing_film_csv(
        self, film_list_df: pd.DataFrame
    ) -> pd.DataFrame:
        should_read_csv = self.check_if_existing_film_csv_exists()

        if not should_read_csv:
            return film_list_df

        existing_film_list = pd.read_csv(self.csv_file_path)

        should_combine_film_lists = self.check_if_existing_csv_is_compatible(
            existing_film_list
        )

        if should_combine_film_lists:
            combined_film_list = self.combine_current_film_list_with_existing_csv(
                existing_film_list, film_list_df
            )

            return combined_film_list

        return film_list_df

    def combine_current_film_list_with_existing_csv(
        self, existing_film_list: pd.DataFrame, current_film_list: pd.DataFrame
    ) -> pd.DataFrame:

        logging.info(
            "Combining existing output file with current list of films "
            "and dropping duplicate film titles..."
        )
        existing_film_list_formatted = existing_film_list.astype(
            self.film_list_df_types
        )

        combined_film_list = (
            pd.concat([current_film_list, existing_film_list_formatted])
            .drop_duplicates(subset=["Parsed film title"], keep="last")
            .reset_index(drop=True)
            .fillna("")  # Fill NAs with empty strings for later dupe handling
        )

        self.stop_process_if_all_films_already_in_existing_csv(combined_film_list)

        return combined_film_list

    def stop_process_if_all_films_already_in_existing_csv(
        self, combined_film_list_df: pd.DataFrame
    ) -> None:
        if all(
            combined_film_list_df["Already on ANT?"].str.contains(
                r"^Duplicate:", regex=True, na=False
            )
        ):
            logging.info(
                "All films have already been searched and are duplicates. "
                "\n\nEnding the process early.\n\n----"
            )
            sys.exit(0)

    def remove_paths_containing_extras_folder(
        self, file_paths: list[Path]
    ) -> list[Path]:
        """
        Remove any file file paths containing Extras as their parent folder.
        """
        cleaned_paths = [path for path in file_paths if path.parent.name != "Extras"]

        return cleaned_paths

    def get_film_sizes_from_file_paths(self, file_paths: list[Path]) -> list[float]:
        film_sizes = [self.get_file_size_from_path(path) for path in file_paths]

        return film_sizes

    def remove_paths_if_unopenable(self, paths):
        """
        If file does not exist or is not openable, remove from paths.
        Warn user if path exceeds 260 characters.
        """
        cleaned_paths = []
        for path in paths:
            if not path.is_file():
                file_name = path.stem
                warning_message = (
                    f"{file_name} could not be opened or does not exist, skipping."
                )
                if len(str(path)) > 260:
                    warning_message += " This may be caused by a file path exceeding 260 characters. Try shortening the folder or file name."
                logging.warning(warning_message)
            else:
                cleaned_paths.append(path)

        return cleaned_paths

    def get_file_size_from_path(self, file_path: Path) -> float:
        file_size_in_bytes = file_path.stat().st_size
        file_size = self.convert_bytes_to_gb(file_size_in_bytes)

        return file_size

    def convert_bytes_to_gb(self, num_in_bytes: int) -> float:
        """
        Convert bytes to correct unit of measurement as a string
        """
        num_in_gb = round((num_in_bytes / 1073741824), 2)

        return num_in_gb

    def get_guessit_info_from_film_paths(
        self, file_paths: list[Path]
    ) -> list[MatchesDict]:
        """
        Use guessit package to extract film information
        into ordered dictionary.
        """
        logging.info("Using Guessit to extract film information, may take a while...")
        guessed_media = [guessit(path) for path in file_paths]
        guessed_films = [media for media in guessed_media if media.get("type") == "movie"]

        logging.info("Finished using Guessit to extract film information")

        return guessed_films

    def get_formatted_titles_from_guessed_films(
        self, guessed_films: list[MatchesDict]
    ) -> list[str]:
        """
        Get film titles from guessit objects, then fix titles missing
        full stops within acronyms
        """
        titles = [
            self.get_film_attribute_from_guessed_film(film, "title")
            for film in guessed_films
        ]
        cleaned_titles = [self.fix_title_if_contains_acronym(title) for title in titles]

        return cleaned_titles

    def get_film_attribute_from_guessed_film(
        self, guessed_film: MatchesDict, attribute: str
    ) -> str:
        """
        Extract the given guessit attribute from a given guessit object.
        """
        try:
            film_attribute = guessed_film[attribute]
            if isinstance(film_attribute, list):
                film_attribute = ", ".join(film_attribute)
        except:
            film_attribute = ""

        return film_attribute

    def get_film_resolutions_from_guessed_films(
        self, guessed_films: list[MatchesDict]
    ) -> list[str]:
        film_resolutions = [
            self.get_film_attribute_from_guessed_film(film, "screen_size")
            for film in guessed_films
        ]

        return film_resolutions

    def get_codec_from_guessed_films(
        self, guessed_films: list[MatchesDict]
    ) -> list[str]:
        film_codecs = [
            self.get_film_attribute_from_guessed_film(film, "video_codec")
            for film in guessed_films
        ]

        film_codecs_cleaned = [re.sub(r"\.", "", codec) for codec in film_codecs]

        return film_codecs_cleaned

    def get_source_from_guessed_films(
        self, guessed_films: list[MatchesDict]
    ) -> list[str]:
        film_sources = [
            self.get_film_attribute_from_guessed_film(film, "source")
            for film in guessed_films
        ]

        film_sources_cleaned = [
            re.sub("Ultra HD Blu-ray", "Blu-ray", source) for source in film_sources
        ]

        return film_sources_cleaned

    def get_release_groups_from_guessed_films(
        self, guessed_films: list[MatchesDict]
    ) -> list[str]:
        release_groups = [
            self.get_film_attribute_from_guessed_film(film, "release_group").lower()
            for film in guessed_films
        ]

        return release_groups

    def fix_title_if_contains_acronym(self, film_title: str) -> str:
        """
        After guessit has extracted film title, fix instances where
        consecutive single letter words contain spaces instead of full stops.
        e.g. L A Confidential -> L.A Confidential -> L.A. Confidential
        e.g. S W A T -> S.W.A.T -> S.W.A.T.
        """
        acronym_spaces_as_full_stops = re.sub(
            r"(?<=\b[A-Za-z]{1})\s(?=[A-Za-z]{1}\b)", ".", film_title
        )

        acronym_suffixed_with_a_full_stop = re.sub(
            r"(?<=\.[A-Za-z])(\s|$)(?=[^\s])", ". ", acronym_spaces_as_full_stops
        )

        acronym_at_end_of_title_suffixed_with_full_stop = re.sub(
            r"(?<=\..$)",
            ".",
            acronym_suffixed_with_a_full_stop,
        )

        return acronym_at_end_of_title_suffixed_with_full_stop

    def create_film_list_dataframe(
        self,
        film_file_paths: list[Path],
        film_sizes: list[float],
        film_titles: list[str],
        film_resolutions: list[str],
        film_codecs: list[str],
        film_sources: list[str],
        film_release_groups: list[str],
    ) -> pd.DataFrame:
        """
        Combine the full file paths and film titles into a
        pandas DataFrame.
        Add empty ANT check column before we attempt to combine an existing
        film list csv, in case one does not exist.
        """
        films_df = pd.DataFrame(
            {
                "Full file path": film_file_paths,
                "Parsed film title": film_titles,
                "Film size (GB)": film_sizes,
                "Resolution": film_resolutions,
                "Codec": film_codecs,
                "Source": film_sources,
                "Release group": film_release_groups,
                "Already on ANT?": np.repeat("", len(film_file_paths)),
                "Info": np.repeat("", len(film_file_paths)),
            }
        ).astype(self.film_list_df_types)

        return films_df

    def check_if_existing_film_csv_exists(self) -> bool:
        if not self.csv_file_path.is_file():
            logging.info(
                "An existing output file (%s) was not found, processing films "
                "from scratch...",
                self.csv_file_path,
            )
            return False

        logging.info("An existing output file (%s) was found.", self.csv_file_path)

        return True

    def check_if_existing_csv_is_compatible(
        self, existing_film_df: pd.DataFrame
    ) -> bool:
        existing_columns = list(existing_film_df.columns)

        if existing_columns != list(self.film_list_df_types.keys()):
            logging.warning(
                "Warning: existing file was created using an old version of ANT upload checker.\n"
                "-------- Existing file is being skipped as it doesn't contain all the required columns.\n"
                "-------- Creating a backup film list from the previous version (%s).\n"
                "-------- This can be deleted if you don't need it.\n",
                self.backup_csv_file_path,
            )

            shutil.copy(self.csv_file_path, self.backup_csv_file_path)

            return False

        return True



import pandas as pd
import numpy as np
import pytest
import logging
from ant_upload_checker.film_searcher import FilmSearcher

LOGGER = logging.getLogger(__name__)


@pytest.fixture
def return_mock_search_for_film_on_ant_not_found(monkeypatch):
    def mockreturn(test_arg, test_arg_2):
        return []

    monkeypatch.setattr(
        "test_film_searcher.FilmSearcher.search_for_film_title_on_ant", mockreturn
    )


@pytest.fixture
def return_mock_search_for_film_on_ant_torrentid(monkeypatch):
    def mockreturn(test_arg, test_arg_2):
        return [{"guid": "url/torrentid=1"}]

    monkeypatch.setattr(
        "test_film_searcher.FilmSearcher.search_for_film_title_on_ant", mockreturn
    )


def test_check_if_films_exist_on_ant(
    return_mock_search_for_film_on_ant_not_found, caplog
):
    # pre dupe check, torrentid links would be skipped from searching
    # post 1.7.0, films are only skipped from searching
    # if start with Duplicate:
    # (as these may have since been uploaded by others)

    caplog.set_level(logging.INFO)

    test_df = pd.DataFrame(
        {
            "Full file path": [
                r"C:\Media\Films\Desert.Flower.2009.1080p.BluRay.x264.DTS-WiKi.mkv",
                r"C:\Media\Films\Dogville (2003) [imdbid-tt0276919] - [Bluray-1080p][DTS-HD MA 5.1][x265]-TAoE.mkv",
                r"C:\Media\Films\Aftersun (2022).mkv",
                r"C:\Media\Films\American Graffiti (1973) [imdbid-tt0069704] - [Remux-1080p Proper][FLAC 2.0][VC1]-KRaLiMaRKo.mkv",
                r"C:\Media\Films\A Man Called Otto (2022) [imdbid-tt7405458] - [Bluray-1080p][EAC3 5.1][x264]-playHD.mkv",
                r"C:\Media\Films\The Last Black Man in San Francisco (2019) [imdbid-tt4353250] - [Bluray-1080p][DTS-HD MA 5.1][x265]-CHD.mkv",
            ],
            "Parsed film title": [
                "Desert Flower",
                "Dogville",
                "Aftersun",
                "American Graffiti",
                "A Man Called Otto",
                "The Last Black Man in San Francisco",
            ],
            "Film size (GB)": [
                18.71,
                11.06,
                17.12,
                26.35,
                16.83,
                6.34,
            ],
            "Resolution": [
                "1080p",
                "1080p",
                "",
                "1080p",
                "1080p",
                "1080p",
            ],
            "Codec": ["H264", "H265", "", "VC-1", "H264", "H265"],
            "Source": ["Blu-ray", "Blu-ray", "", "", "Blu-ray", "Blu-ray"],
            "Release group": ["wiki", "taoe", "", "kralimark", "playhd", "chd"],
            "Already on ANT?": [
                "Uploadable - potentially",
                "Uploadable",
                "Duplicate - potentially",
                "Duplicate - partial",
                "Duplicate",
                "Banned",
            ],
            "Info": [
                "Film not found on ANT - does not already exist, or title failed to match",
                "A film with 1080p/H265/Blu-ray does not already exist. test_link",
                "On ANT, but could not dupe check (could not extract resolution/codec/media from filename). test_link",
                "A film with 1080p/VC-1 already exists. Could not extract and check media from filename. test_link",
                "A film with 1080p/H264/Blu-ray already exists: test_link",
                "Release group 'chd' is banned from ANT - do not upload",
            ],
        }
    )

    fs = FilmSearcher(test_df, "test_api_key")
    actual_df = fs.check_if_films_exist_on_ant()
    expected_df = pd.DataFrame(
        {
            "Full file path": [
                r"C:\Media\Films\A Man Called Otto (2022) [imdbid-tt7405458] - [Bluray-1080p][EAC3 5.1][x264]-playHD.mkv",
                r"C:\Media\Films\Aftersun (2022).mkv",
                r"C:\Media\Films\American Graffiti (1973) [imdbid-tt0069704] - [Remux-1080p Proper][FLAC 2.0][VC1]-KRaLiMaRKo.mkv",
                r"C:\Media\Films\Desert.Flower.2009.1080p.BluRay.x264.DTS-WiKi.mkv",
                r"C:\Media\Films\Dogville (2003) [imdbid-tt0276919] - [Bluray-1080p][DTS-HD MA 5.1][x265]-TAoE.mkv",
                r"C:\Media\Films\The Last Black Man in San Francisco (2019) [imdbid-tt4353250] - [Bluray-1080p][DTS-HD MA 5.1][x265]-CHD.mkv",
            ],
            "Parsed film title": [
                "A Man Called Otto",
                "Aftersun",
                "American Graffiti",
                "Desert Flower",
                "Dogville",
                "The Last Black Man in San Francisco",
            ],
            "Film size (GB)": [
                16.83,
                17.12,
                26.35,
                18.71,
                11.06,
                6.34,
            ],
            "Resolution": [
                "1080p",
                "",
                "1080p",
                "1080p",
                "1080p",
                "1080p",
            ],
            "Codec": ["H264", "", "VC-1", "H264", "H265", "H265"],
            "Source": ["Blu-ray", "", "", "Blu-ray", "Blu-ray", "Blu-ray"],
            "Release group": ["playhd", "", "kralimark", "wiki", "taoe", "chd"],
            "Already on ANT?": [
                "Duplicate",
                "Duplicate - potentially",
                "Duplicate - partial",
                "Uploadable - potentially",
                "Uploadable",
                "Banned",
            ],
            "Info": [
                "A film with 1080p/H264/Blu-ray already exists: test_link",
                "On ANT, but could not dupe check (could not extract resolution/codec/media from filename). test_link",
                "A film with 1080p/VC-1 already exists. Could not extract and check media from filename. test_link",
                "Film not found on ANT - does not already exist, or title failed to match",
                "A film with 1080p/H265/Blu-ray does not already exist. test_link",
                "Release group 'chd' is banned from ANT - do not upload",
            ],
            "Should skip": [True, True, True, False, False, True],
            "API response": [[], [], [], [], [], []],
        }
    )

    assert (
        "Skipping 4 films already found on ANT in the previous output file..."
        in caplog.text
    )
    pd.testing.assert_frame_equal(actual_df, expected_df)


@pytest.mark.parametrize("film_title", ["A film", "A film with and in the title"])
def test_check_if_film_exists_on_ant_false(
    return_mock_search_for_film_on_ant_not_found, caplog, film_title
):
    caplog.set_level(logging.INFO)

    fs = FilmSearcher("test", "test_api_key")
    actual_return = fs.check_if_film_exists_on_ant(film_title)

    assert f"Searching for {film_title}" in caplog.text
    assert actual_return == []
    assert "--- Not found on ANT ---" in caplog.text

    if "and" in film_title:
        assert "Searching for A film with & in the title as well" in caplog.text


@pytest.mark.parametrize("film_title", ["A film"])
def test_check_if_film_exists_on_ant_true(
    return_mock_search_for_film_on_ant_torrentid, caplog, film_title
):
    caplog.set_level(logging.INFO)

    fs = FilmSearcher("test", "test_api_key")
    actual_return = fs.check_if_film_exists_on_ant(film_title)

    assert f"Searching for {film_title}" in caplog.text
    assert actual_return == [{"guid": "url/torrentid=1"}]
    assert "--- Not found on ANT ---" not in caplog.text


def test_replace_word_and_re_search(
    return_mock_search_for_film_on_ant_not_found, caplog
):
    caplog.set_level(logging.INFO)
    fs = FilmSearcher("test", "test_api_key")

    test_film = "Test film biscuit"
    test_regex = "biscuit"
    test_replacement = "jaffa"

    actual_return = fs.replace_word_and_re_search(
        test_film, test_regex, test_replacement
    )

    assert actual_return == []
    assert "Searching for Test film jaffa as well..." in caplog.text


films_four_numbers = {
    "1208 East of Bucharest": "Searching for 12:08 East of Bucharest as well...",
    "Test film 1508": "Searching for Test film 15:08 as well...",
    "Film 1000 test": "Searching for Film 10:00 test as well...",
}


@pytest.mark.parametrize("test_film", films_four_numbers)
def test_search_for_film_if_contains_potential_time_true(
    return_mock_search_for_film_on_ant_not_found, caplog, test_film
):
    caplog.set_level(logging.INFO)
    fs = FilmSearcher("test", "test_api_key")

    actual_return = fs.search_for_film_if_contains_potential_date_or_time(
        test_film, format="time"
    )

    assert actual_return == []
    expected_log_info = films_four_numbers[test_film]
    assert expected_log_info in caplog.text
    assert "Film title may contain a date or time" in caplog.text


films_five_numbers = ["12345 test film", "Test film 12345", "Film 12345 test"]


@pytest.mark.parametrize("test_film", films_five_numbers)
def test_search_for_film_if_contains_potential_time_false(
    return_mock_search_for_film_on_ant_torrentid, caplog, test_film
):
    # Film search mock return is the torrentid. This should never be returned
    # If the film does not contain 4 numbers, as the search is skipped.
    caplog.set_level(logging.INFO)

    fs = FilmSearcher("test", "test_api_key")

    actual_return = fs.search_for_film_if_contains_potential_date_or_time(
        test_film, format="time"
    )

    assert actual_return == []
    assert "Film title may contain a date or time" not in caplog.text


films_with_dates_numbers = {
    "77 One Day in London": "Searching for 7/7 One Day in London as well...",
    "Test film 11": "Searching for Test film 1/1 as well...",
    "Film 89 test": "Searching for Film 8/9 test as well...",
    "911 film": "Searching for 9/11 film as well...",
    "Fahrenheit 911": "Searching for Fahrenheit 9/11 as well...",
    "Film 112 test": "Searching for Film 1/12 test as well...",
}


@pytest.mark.parametrize("test_film", films_with_dates_numbers)
def test_search_for_film_if_contains_potential_date_true(
    return_mock_search_for_film_on_ant_not_found, caplog, test_film
):
    caplog.set_level(logging.INFO)
    fs = FilmSearcher("test", "test_api_key")

    actual_return = fs.search_for_film_if_contains_potential_date_or_time(
        test_film, format="date"
    )

    assert actual_return == []
    expected_log_info = films_with_dates_numbers[test_film]
    assert expected_log_info in caplog.text
    assert "Film title may contain a date or time" in caplog.text


films_with_numbers_not_dates = [
    "7777 One Day in London",
    "Test film 1112",
    "Film 8910 test",
    "9 film",
    "Fahrenheit 91144",
    "Film 112444 test",
]


@pytest.mark.parametrize("test_film", films_five_numbers)
def test_search_for_film_if_contains_potential_date_false(
    return_mock_search_for_film_on_ant_torrentid, caplog, test_film
):
    # Film search mock return is the torrentid. This should never be returned
    # If the film does not contain 4 numbers, as the search is skipped.
    caplog.set_level(logging.INFO)

    fs = FilmSearcher("test", "test_api_key")

    actual_return = fs.search_for_film_if_contains_potential_date_or_time(
        test_film, format="date"
    )

    assert actual_return == []
    assert "Film title may contain a date or time" not in caplog.text


films_with_alternate_titles = {
    "Alphaville une etrange aventure de Lemmy Caution AKA Alphaville": (
        "Alphaville une etrange aventure de Lemmy Caution",
        "Alphaville",
    ),
    "Title 1 aka Title 2": ("Title 1", "Title 2"),
}


@pytest.mark.parametrize("test_film", films_with_alternate_titles)
def test_search_for_film_if_contains_aka_true(
    return_mock_search_for_film_on_ant_not_found, caplog, test_film
):
    caplog.set_level(logging.INFO)

    fs = FilmSearcher("test", "test_api_key")
    actual_return = fs.search_for_film_if_contains_aka(test_film)

    assert actual_return == []
    assert "Film title may contain an alternate title" in caplog.text
    assert (
        f"Searching for {films_with_alternate_titles[test_film][0]} as well..."
        in caplog.text
    )
    assert (
        f"Searching for {films_with_alternate_titles[test_film][1]} as well..."
        in caplog.text
    )


films_with_no_alternate_title = [
    "Aka Test film",
    "Test film aka",
    "Film with baka in it",
]


@pytest.mark.parametrize("test_film", films_with_no_alternate_title)
def test_search_for_film_if_contains_aka_false(
    return_mock_search_for_film_on_ant_not_found, caplog, test_film
):
    caplog.set_level(logging.INFO)

    fs = FilmSearcher("test", "test_api_key")
    actual_return = fs.search_for_film_if_contains_aka(test_film)

    assert actual_return == []
    assert "Film title may contain an alternate title" not in caplog.text
    assert f"Searching for Test film as well" not in caplog.text



import pytest
from ant_upload_checker import setup_functions
from pathlib import Path
import inquirer


def test_return_as_path_if_valid(tmp_path):
    temp_path_str = str(tmp_path)
    actual_return = setup_functions.return_as_path_if_valid(temp_path_str)
    assert isinstance(actual_return, Path)


def test_return_as_path_if_valid_bad_file_path():
    test_incorrect_path = "C:/Media/Fake_path"
    with pytest.raises(
        ValueError,
        match="Folder path 'C:/Media/Fake_path' does not exist. Please try a different path.",
    ):
        setup_functions.return_as_path_if_valid(test_incorrect_path)


def test_return_as_path_if_valid_none():
    with pytest.raises(
        ValueError,
        match="No folder path was selected in the dialog box, please re-run",
    ):
        setup_functions.return_as_path_if_valid(None)


TEST_INPUT_PATHS = [["Films", "Old films"], ["Single film folder"]]


@pytest.mark.parametrize("test_paths", TEST_INPUT_PATHS)
def test_get_user_input_folders(monkeypatch, tmp_path, test_paths):
    temp_paths = [tmp_path / path for path in test_paths]
    [folder.mkdir() for folder in temp_paths]

    temp_paths_str = [str(temp_path) for temp_path in temp_paths]

    def mock_prompt(_):
        # Note empty space that should be removed
        return {"input_folders": ", ".join(temp_paths_str)}

    monkeypatch.setattr(inquirer, "prompt", mock_prompt)

    actual_return = setup_functions.get_user_input_folders()
    expected_return = [str(folder) for folder in temp_paths]

    assert actual_return == expected_return


def test_get_user_output_folder(monkeypatch, tmp_path):
    temp_path = tmp_path / "Media"
    temp_path.mkdir()

    temp_path_str = str(temp_path)

    def mock_prompt(_):
        # Empty space before path should be removed
        return {"output_folder": f" {temp_path_str}"}

    monkeypatch.setattr(inquirer, "prompt", mock_prompt)

    actual_return = setup_functions.get_user_output_folder()
    expected_return = temp_path_str

    assert actual_return == expected_return


def test_get_user_info_api_key_empty(monkeypatch):
    with pytest.raises(ValueError, match="No API key was set - please restart"):

        def mock_prompt(_):
            return {"api_key": ""}

        monkeypatch.setattr(inquirer, "prompt", mock_prompt)

        setup_functions.get_user_info_api_key()
